SystemOrganization addCategory: #'Cutie-Helvetia-Linda'!

!BlockContext methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/11/2009 15:47'!
lindaMatch: anObject
	"Use a block condition to compare."

	^ self value: anObject! !

QQTestCase subclass: #CULindaTest
	instanceVariableNames: 'space instVar'
	classVariableNames: 'ClassVar DontCare'
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Linda'!

!CULindaTest class methodsFor: 'initialization' stamp: 'lr 9/2/2010 15:09'!
initialize
	DontCare := CUDontCareBinding new! !

!CULindaTest class methodsFor: 'transformations' stamp: 'lr 9/2/2010 15:13'!
transformation
	<transform>

	^ CHTreePattern new
		expression: '`var asLindaBinding' do: [ :context |
			context scopedTree swapWith: ``(CUPluggableBinding
				on: [ :lindaValue | `,(context scopedTree receiver) := lindaValue ]) ];
		yourself! !

!CULindaTest methodsFor: 'utilities' stamp: 'lr 2/23/2009 13:41'!
fibonacci: anInteger
	"Calculates fibonacci numbers."

	| result |

	" try to read a cached version "
	(space read: { 'fib'. anInteger. result asLindaBinding } ifNone: [ nil ]) isNil
		ifFalse: [ ^ result ].
		
	" set beginning of recursion "
	anInteger < 2 ifTrue: [
		space write: { 'fib'. anInteger. 1 }.
		^ 1 ].
	
	" recurse "
	[ space write: { 'fib'. anInteger. (self fibonacci: anInteger - 1) + (self fibonacci: anInteger - 2) } ]
		fork.

	" wait for result "
	space read: { 'fib'. anInteger. result asLindaBinding }.
	^ result! !

!CULindaTest methodsFor: 'utilities' stamp: 'lr 2/23/2009 13:41'!
philosophersLife
	"The dining philosopher process."
	| total name ticket fork1 fork2 |

	" determine the total number and my name "
	space read: { 'total'. total asLindaBinding }.
	space take: { 'philosopher'. name asLindaBinding }.
	
	" philosophers eat 10 times, then they are done "
	10 timesRepeat: [
		Transcript show: 'Philosopher ' , name asString , ' is thinking.'; cr.
		(Delay forMilliseconds: 500 atRandom) wait.
		
		ticket := space take: { 'ticket' }.
		fork1 := space take: { 'fork'. name }.
		fork2 := space take: { 'fork'. (name \\ total) + 1 }.
		
		Transcript show: 'Philosopher ' , name asString , ' is eating.'; cr.
		(Delay forMilliseconds: 500 atRandom) wait.

		space write: fork1; write: fork2; write: ticket ].
	
	Transcript show: 'Philosopher ' , name asString , ' is done.'; cr! !

!CULindaTest methodsFor: 'utilities' stamp: 'lr 2/23/2009 13:41'!
philosophersTableOf: anInteger
	"Runs the dining philosopher problem with the philosophers given in anArray."
	" CULindaTest new setUp; philosophersTableOf: 5 "

	" remember the number of participants "
	space write: { 'total'. anInteger }.	
	
	" create the philosophers and give them each a fork "
	1 to: anInteger do: [ :index |
		space write: { 'fork'. index }.
		space write: { 'philosopher'. index }.
		index < anInteger
			ifTrue: [ space write: { 'ticket' } ] ].
		
	" start the philosophers "
	1 to: anInteger do: [ :index |
		[ self philosophersLife ]
			forkAt: Processor userBackgroundPriority 
			named: 'Philosopher ' , index asString ]! !

!CULindaTest methodsFor: 'running' stamp: 'lr 2/23/2009 13:40'!
setUp
	space := CULindaSpace new! !

!CULindaTest methodsFor: 'testing-bindings' stamp: 'lr 2/23/2009 13:40'!
testBindDontCare
	space write: { 'testBindDontCare'. true }.
	space read: { 'testBindDontCare'. DontCare }! !

!CULindaTest methodsFor: 'testing-bindings' stamp: 'lr 2/23/2009 13:40'!
testBindGlobal
	ClassVar := false.
	space write: { 'testGlobalBinding'. true }.
	space read: { 'testGlobalBinding'. ClassVar asLindaBinding }.
	self assert: ClassVar! !

!CULindaTest methodsFor: 'testing-bindings' stamp: 'lr 2/23/2009 13:40'!
testBindInstance
	instVar := false.
	space write: { 'testInstanceBinding'. true }.
	space read: { 'testInstanceBinding'. instVar asLindaBinding }.
	self assert: instVar! !

!CULindaTest methodsFor: 'testing-bindings' stamp: 'lr 9/2/2010 15:12'!
testBindTemp
	| temp |
	temp := false.
	space write: { 'testTempBinding'. true }.
	space read: { 'testTempBinding'. temp asLindaBinding }.
	self assert: temp! !

!CULindaTest methodsFor: 'testing-scenario' stamp: 'lr 2/23/2009 13:41'!
testFibonacci
	self assert: (self fibonacci: 5) = 8.
	self assert: (self fibonacci: 8) = 34.
	self assert: (self fibonacci: 12) = 233.
	self assert: (self fibonacci: 30) = 1346269! !

!CULindaTest methodsFor: 'testing-matching' stamp: 'lr 2/23/2009 13:41'!
testMatchBlock
	space write: #(1); write: #(2); write: #(3).

	self assert: (space scan: { [ :a | a even ] }) size = 1.
	self assert: (space scan: { [ :a | a odd ] }) size = 2! !

!CULindaTest methodsFor: 'testing-matching' stamp: 'lr 2/23/2009 13:41'!
testMatchClass
	space write: #('foo'); write: #(123); write: #(45.6); write: #(false).

	self assert: (space scan: { String }) size = 1.
	self assert: (space scan: { Integer }) size = 1.
	self assert: (space scan: { Float }) size = 1.
	self assert: (space scan: { Boolean }) size = 1.
	self assert: (space scan: { UndefinedObject }) isEmpty! !

!CULindaTest methodsFor: 'testing-matching' stamp: 'lr 2/23/2009 13:41'!
testMatchLiteral
	space write: #('foo'); write: #(123); write: #(45.6); write: #(false).
	
	self assert: (space scan: #('foo')) size = 1.
	self assert: (space scan: #(123)) size = 1.
	self assert: (space scan: #(45.6)) size = 1.
	self assert: (space scan: #(false)) size = 1.
	self assert: (space scan: #(true)) isEmpty! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testRead
	| value semaphore process |
	value := false.
	semaphore := Semaphore new.
	process := [ space read: { value asLindaBinding }. semaphore signal ]
		newProcess.
	process resume.
	self deny: value.
	self deny: process isTerminated.
	space write: #(true).
	semaphore wait.
	self assert: value! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testReadIfNone
	| result value |
	result := space read: { value asLindaBinding } ifNone: [ true ].
	self assert: result.
	self assert: value isNil.
	space write: #(1).
	result := space read: { value asLindaBinding } ifNone: [ false ].
	self assert: result = #(1).
	self assert: value = 1.
	result := space read: { value asLindaBinding } ifNone: [ false ].
	self assert: result = #(1).
	self assert: value = 1! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testScan
	self assert: (space scan: #()) isEmpty.
	self assert: (space scan: #(1)) isEmpty.
	space write: #(); write: #(1).
	self assert: (space scan: #()) size = 1.
	self assert: (space scan: #(1)) size = 1
	! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testScanDo
	space scan: #() do: [ :each | self fail ].
	space write: #(1).
	space scan: #() do: [ :each | self fail ].
	space scan: #(1) do: [ :each | self assert: each first = 1 ]! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testTake
	| value semaphore process |
	value := false.
	semaphore := Semaphore new.
	process := [ space take: { value asLindaBinding }. semaphore signal ]
		newProcess.
	process resume.
	self deny: value.
	self deny: process isTerminated.
	space write: #(true).
	semaphore wait.
	self assert: value.
	self assert: (space 
		take: { DontCare }
		ifNone: [ true ])! !

!CULindaTest methodsFor: 'testing' stamp: 'lr 2/23/2009 13:40'!
testTakeIfNone
	| result value |
	result := space take: { value asLindaBinding } ifNone: [ true ].
	self assert: result.
	self assert: value isNil.
	space write: #(1).
	result := space take: { value asLindaBinding } ifNone: [ false ].
	self assert: result = #(1).
	self assert: value = 1.
	result := space take: { value asLindaBinding } ifNone: [ false ].
	self deny: result.
	self assert: value = 1! !

!BlockClosure methodsFor: '*cutie-helvetia-linda' stamp: 'lr 5/18/2009 15:02'!
lindaMatch: anObject
	"Use a block condition to compare."

	^ self value: anObject! !

Object subclass: #CULindaBinding
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Linda'!

CULindaBinding subclass: #CUDontCareBinding
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Linda'!

!CUDontCareBinding methodsFor: 'actions' stamp: 'lr 2/16/2009 11:11'!
lindaBind: anObject
	^ anObject! !

!CULindaBinding methodsFor: 'actions' stamp: 'lr 2/13/2009 13:58'!
lindaBind: anObject
	self subclassResponsibility! !

!CULindaBinding methodsFor: 'testing' stamp: 'lr 2/11/2009 15:47'!
lindaMatch: anObject
	"A binding matches everything."

	^ true! !

CULindaBinding subclass: #CUPluggableBinding
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Linda'!

!CUPluggableBinding class methodsFor: 'instance creation' stamp: 'lr 9/2/2010 14:10'!
on: aBlock
	^ self new block: aBlock! !

!CUPluggableBinding methodsFor: 'accessing' stamp: 'lr 9/2/2010 14:10'!
block
	^ block! !

!CUPluggableBinding methodsFor: 'accessing' stamp: 'lr 9/2/2010 14:10'!
block: aBlock
	block := aBlock! !

!CUPluggableBinding methodsFor: 'actions' stamp: 'lr 9/2/2010 14:10'!
lindaBind: anObject
	^ block value: anObject! !

Object subclass: #CULindaSpace
	instanceVariableNames: 'monitor tuples'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cutie-Helvetia-Linda'!

!CULindaSpace methodsFor: 'initialization' stamp: 'lr 2/13/2009 15:19'!
initialize
	monitor := Monitor new.
	tuples := OrderedCollection new! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 15:12'!
read: aTuple
	"Blocking, non-destructive read from the tuple space."
	
	^ self read: aTuple ifNone: nil! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 15:15'!
read: aTuple ifNone: aBlock
	"Non-blocking, non-destructive read from the tuple space."
	
	monitor critical: [ [
		1 to: tuples size do: [ :index |
			(aTuple lindaMatch: (tuples at: index))
				ifTrue: [ ^ aTuple lindaBind: (tuples at: index) ] ].
		aBlock isNil
			ifTrue: [ monitor wait ]
			ifFalse: [ ^ aBlock value ] ] 
				repeat ]! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/11/2009 16:49'!
scan: aTuple
	"Scan the complete tuple space and answer all matching tuples."
	
	| stream |
	stream := WriteStream on: (Array new: 16).
	self scan: aTuple do: [ :tuple | stream nextPut: tuple ].
	^ stream contents! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 14:56'!
scan: aTuple do: aBlock
	"Scan the complete tuple space and evaluate aBlock for all matching tuples."
	
	| tuple |
	monitor critical: [
		1 to: tuples size do: [ :index |
			(aTuple lindaMatch: (tuple := tuples at: index))
				ifTrue: [ aBlock value: (aTuple lindaBind: tuple) ] ] ]! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 15:12'!
take: aTuple
	"Blocking, destructive read from the tuple space."
	
	^ self take: aTuple ifNone: nil! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 15:20'!
take: aTuple ifNone: aBlock
	"Non-blocking, destructive read from the tuple space."
	
	monitor critical: [ [
		1 to: tuples size do: [ :index |
			(aTuple lindaMatch: (tuples at: index))
				ifTrue: [ ^ aTuple lindaBind: (tuples removeAt: index) ] ].
		aBlock isNil
			ifTrue: [ monitor wait ]
			ifFalse: [ ^ aBlock value ] ]
				repeat ]! !

!CULindaSpace methodsFor: 'public' stamp: 'lr 2/13/2009 14:57'!
write: aTuple
	"Add a tuple into a space."
	
	monitor critical: [
		tuples add: aTuple.
		monitor signalAll ]! !

!Object methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/13/2009 16:25'!
asLindaBinding
	self error: 'There is something wrong with your compiler.'! !

!Object methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/11/2009 15:56'!
lindaBind: anObject
	"Bind the value anObject to the receiver. Does nothing by default."
	
	^ anObject! !

!Object methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/11/2009 15:47'!
lindaMatch: anObject
	"Normally the default Smalltalk comparator is used."
	
	^ self = anObject! !

!Behavior methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/13/2009 16:26'!
lindaMatch: anObject
	"Use a block condition to compare."

	^ anObject isKindOf: self! !

!SequenceableCollection methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/11/2009 15:56'!
lindaBind: anObject
	"Bind the value anObject to the receiver."
	
	| result |
	result := self class new: self size.
	1 to: self size do: [ :index |
		result at: index put: ((self at: index) 
			lindaBind: (anObject at: index)) ].
	^ result! !

!SequenceableCollection methodsFor: '*cutie-helvetia-linda' stamp: 'lr 2/11/2009 15:50'!
lindaMatch: anObject
	"This is the tuple comparison. Dispatch to the individual elements of the other collection."

	| other |
	(anObject isCollection and: [ self size = anObject size ])
		ifFalse: [ ^ false ].
	other := anObject asArray.
	1 to: self size do: [ :index |
		((self at: index) lindaMatch: (other at: index))
			ifFalse: [ ^ false ] ].
	^ true! !
CULindaTest initialize!
